<?php 

namespace WebServiceCall\Service;

use WebServiceCall\Service\Interfaces\IPayments;

use WebServiceCall\Repository\DB\Payments as PaymentsRepo;
use WebServiceCall\Repository\DB\ProdutoAdq as ProdutoAdqRepo;
use WebServiceCall\Repository\DB\Adquirente as AdquirenteRepo;

use WebServiceCall\Repository\WS\Data\PaymentsParams;

class Payments extends ABaseWSService implements IPayments {
    private const SERVICE_NAME = 'Pagamentos';

    private $produtoAdqRepo;
    private $paymentsRepo;
    private $adquirenteRepo;
    
    public function __construct($connParams, $initialDate, $endDate) {
        parent::__construct($connParams, $initialDate, $endDate, self::SERVICE_NAME);
        
        $this->setPaymentsRepo(new PaymentsRepo ());
        $this->setProdutoAdqRepo(new ProdutoAdqRepo ());
        $this->setAdquirenteRepo(new AdquirenteRepo ());
    }
    
    private function setPaymentsRepo($paymentsRepo) {
        $this->paymentsRepo = $paymentsRepo; 
    }
    
    private function setProdutoAdqRepo($produtoAdqRepo) {
        $this->produtoAdqRepo = $produtoAdqRepo; 
    }
    
    private function setAdquirenteRepo($adquirenteRepo) {
        $this->adquirenteRepo = $adquirenteRepo; 
    }
    
    // @Override
    public function importPaymentsFromWebService() {
        parent::importDataFromWebService();
    }

    // @Override
    protected function getSoapParams($connParam) {
        return new PaymentsParams(
            $connParam->getIdEmpresa(), 
            $connParam->getPass(), 
            $this->getInitialDate(), 
            $this->getFinalDate()
        );
    }
    
    // @Override
    protected function setInsertParams($payment) {
        return $this->setPaymentParams($payment);
    }
    
    // @Override
    protected function saveDataInDB($payment) {
        return $this->savePayment($payment);
    }

    // @Override
    protected function commit() {
        return $this->paymentsRepo->commit();
    }

    // @Override
    protected function rollback() {
        return $this->paymentsRepo->rollback();
    }
    
    private function savePayment($payment) {
        $paymentParams = [];
        
        // Chama o método que valida se o pagamento atual já está inserido
        $isPaymentInserted = $this->isPaymentInserted($payment);

        if (!$isPaymentInserted) {
            // Os campos Pago e Cancelamento do relatório são ignorados 
            // pois a função deles foi incorporada em uma única coluna (IDSTATUS) no nosso sistema. 
            // O campo CNPJ não é parte das colunas da tabela RETORIGVENDA
            foreach ($payment as $key => $value) {
                if ($this->isAValidInsertPaymentKey($key) ) {
                    $paymentParams[$key] = $value;
                }
            }
            
            return $this->paymentsRepo->insertPayment($paymentParams);
        }
        
        return true; // No errors, but the register already exists.
    }
    
    private function isPaymentInserted ($payment) {
        $nsu = $payment[':NSU'];
        $cdLoteVenda = $payment[':ResumoVenda'];
        $autorizacao = $payment[':Autorizacao'];
        $dtCredito = $this->formatDTCREDITO($payment[':DataPagamento']);
        $vrVenda = $payment[':ValorBruto'];
        
        return $this->paymentsRepo->isPaymentInserted($nsu, $cdLoteVenda, $autorizacao, $dtCredito, $vrVenda);
    }

    private function isAValidInsertPaymentKey($key) {
        return  !($key === ':IdentificadorPagamento' ||
                  $key === ':Cnpj' ||
                  $key === ':IdStatus' ||
                  $key === ':Status' ||
                  $key === ':Divergencias' ||
                  $key === ':ValorLiquido');
    }

    private function setPaymentParams($payment) {
        $payment[":NRADQUIRENTE"] = $this->adquirenteRepo->getNextAdquirenteID($payment[':Adquirente']); 
        
        // Define o sequencial do código do produto baseado no campo de nome do produto do relatório.
        $produtoAdq = $this->produtoAdqRepo->getProdutoAdqByTipoProduto($payment[':TipoProduto']);
        
        $payment[':NRPRODUTOADQ'] = $produtoAdq['NRPRODUTOADQ'];
        $payment[':CDPRODUTOADQ'] = $produtoAdq['CDPRODUTOADQ'];

        if ($this->isProdutoAdqNotFound($payment)) {
            throw new \Exception("PAGAMENTO -> Não foi encontrado produto para a filtragem. Produto -> " . $payment[':TipoProduto']);
        }
        
        // Calcula o valor líquido a ser pago pela adquirente e a taxa cobrada pela venda
        $payment[":ValorBruto"]    = floatval(str_replace(",", ".", $payment[':ValorBruto']));
        $payment[":Taxa"]          = floatval(str_replace(",", ".", $payment[':Taxa']));
        $payment[":ValorLiquido"]  = floatval(str_replace(",", ".", $payment[':ValorLiquido']));
        $payment[':VRLIQVENDA']    = str_replace(".", ",", floatval($payment[":ValorBruto"] - $payment[":ValorBruto"] * ($payment[":Taxa"] / 100)));
        $payment[':VRCOBRADOTAXA'] = str_replace(".", ",", floatval($payment[":ValorBruto"] - $payment[":ValorLiquido"]));
        $payment[":ValorBruto"]    = str_replace(".", ",", $payment[':ValorBruto']);
        $payment[":Taxa"]          = str_replace(".", ",", $payment[':Taxa']);
        $payment[":ValorLiquido"]  = str_replace(".", ",", $payment[':ValorLiquido']);
        
        // Converte os valores do relatório para caracteres, de forma que esses valores sejam compatíveis com os valores
        // possíveis para as colunas no BD
        $payment[':Antecipacao'] = $payment[':Antecipacao'] === 'Sim' ? 'S' : 'N';

        switch ($payment[':IdTipoTransacao']) {
            case '1':
                $payment[':IdTipoTransacao'] = 'V'; /* Venda */
                break;
            case '2':
                $payment[':IdTipoTransacao'] = 'P'; /* Pagamentos */
                break; 
            case '3':
                $payment[':IdTipoTransacao'] = 'C'; /* Cancelamentos */
                break;
            case '4':
                $payment[':IdTipoTransacao'] = 'D'; /* Débitos */
                break;
        }
        
        return $payment;
    }
     
    private function isProdutoAdqNotFound($payment) {
        return ( (!$payment[':NRPRODUTOADQ'] || !$payment[':CDPRODUTOADQ']) &&
                  ($payment[':Adquirente'] !== 'VR') && ($payment[':Autorizacao'] || $payment[':NSU']) );
    }
    
    // Garante que o formato do DTCREDITO será sempre DD/MM/YYYY
    private function formatDTCREDITO($dtCredito) {
	    $dtCredito = explode('/', $dtCredito);
	    $year = $dtCredito[2];
	    if(strlen($year) != 4 ){
	       $currYear = str_split(date('Y'));
	       $currYear = $currYear[0].$currYear[1].'0'.'0';
	       $year += $currYear;
	    }
	    $dtCredito[2] = $year;
	    $dtCredito = $dtCredito[0].'/'.$dtCredito[1].'/'.$dtCredito[2];
	    return $dtCredito;
    }
}